@@ -12,20 +12,11 @@ LTTNG_TRACEPOINT_EVENT(gpio_direction,
 
 	TP_ARGS(gpio, in, err),
 
-	TP_STRUCT__entry(
-		__field(unsigned, gpio)
-		__field(int, in)
-		__field(int, err)
-	),
-
-	TP_fast_assign(
-		tp_assign(gpio, gpio)
-		tp_assign(in, in)
-		tp_assign(err, err)
-	),
-
-	TP_printk("%u %3s (%d)", __entry->gpio,
-		__entry->in ? "in" : "out", __entry->err)
+	TP_FIELDS(
+		ctf_integer(unsigned, gpio, gpio)
+		ctf_integer(int, in, in)
+		ctf_integer(int, err, err)
+	)
 )
 
 LTTNG_TRACEPOINT_EVENT(gpio_value,
@@ -34,20 +25,11 @@ LTTNG_TRACEPOINT_EVENT(gpio_value,
 
 	TP_ARGS(gpio, get, value),
 
-	TP_STRUCT__entry(
-		__field(unsigned, gpio)
-		__field(int, get)
-		__field(int, value)
-	),
-
-	TP_fast_assign(
-		tp_assign(gpio, gpio)
-		tp_assign(get, get)
-		tp_assign(value, value)
-	),
-
-	TP_printk("%u %3s %d", __entry->gpio,
-		__entry->get ? "get" : "set", __entry->value)
+	TP_FIELDS(
+		ctf_integer(unsigned, gpio, gpio)
+		ctf_integer(int, get, get)
+		ctf_integer(int, value, value)
+	)
 )
 
 #endif /* if !defined(LTTNG_TRACE_GPIO_H) || defined(TRACE_HEADER_MULTI_READ) */
