@@ -24,23 +24,12 @@ LTTNG_TRACEPOINT_EVENT_CLASS(regmap_reg,
 
 	TP_ARGS(map, reg, val),
 
-	TP_STRUCT__entry(
-		__string(	name,		regmap_name(map))
-		__field(	unsigned int,	reg		)
-		__field(	unsigned int,	val		)
-	),
-
-	TP_fast_assign(
-		tp_strcpy(name, regmap_name(map))
-		tp_assign(reg, reg)
-		tp_assign(val, val)
-	),
-
-	TP_printk("%s reg=%x val=%x", __get_str(name),
-		  (unsigned int)__entry->reg,
-		  (unsigned int)__entry->val)
+	TP_FIELDS(
+		ctf_string(name, regmap_name(map))
+		ctf_integer(unsigned int, reg, reg)
+		ctf_integer(unsigned int, val, val)
+	)
 )
-
 LTTNG_TRACEPOINT_EVENT_INSTANCE(regmap_reg, regmap_reg_write,
 
 	TP_PROTO(struct regmap *map, unsigned int reg,
@@ -71,21 +60,11 @@ LTTNG_TRACEPOINT_EVENT_CLASS(regmap_block,
 
 	TP_ARGS(map, reg, count),
 
-	TP_STRUCT__entry(
-		__string(	name,		regmap_name(map))
-		__field(	unsigned int,	reg		)
-		__field(	int,		count		)
-	),
-
-	TP_fast_assign(
-		tp_strcpy(name, regmap_name(map))
-		tp_assign(reg, reg)
-		tp_assign(count, count)
-	),
-
-	TP_printk("%s reg=%x count=%d", __get_str(name),
-		  (unsigned int)__entry->reg,
-		  (int)__entry->count)
+	TP_FIELDS(
+		ctf_string(name, regmap_name(map))
+		ctf_integer(unsigned int, reg, reg)
+		ctf_integer(int, count, count)
+	)
 )
 
 LTTNG_TRACEPOINT_EVENT_INSTANCE(regmap_block, regmap_hw_read_start,
@@ -116,27 +95,20 @@ LTTNG_TRACEPOINT_EVENT_INSTANCE(regmap_block, regmap_hw_write_done,
 	TP_ARGS(map, reg, count)
 )
 
-LTTNG_TRACEPOINT_EVENT(regcache_sync,
+LTTNG_TRACEPOINT_EVENT_MAP(regcache_sync,
+
+	regmap_regcache_sync,
 
 	TP_PROTO(struct regmap *map, const char *type,
 		 const char *status),
 
 	TP_ARGS(map, type, status),
 
-	TP_STRUCT__entry(
-		__string(       name,           regmap_name(map))
-		__string(	status,		status		)
-		__string(	type,		type		)
-	),
-
-	TP_fast_assign(
-		tp_strcpy(name, regmap_name(map))
-		tp_strcpy(status, status)
-		tp_strcpy(type, type)
-	),
-
-	TP_printk("%s type=%s status=%s", __get_str(name),
-		  __get_str(type), __get_str(status))
+	TP_FIELDS(
+		ctf_string(name, regmap_name(map))
+		ctf_string(status, status)
+		ctf_string(type, type)
+	)
 )
 
 LTTNG_TRACEPOINT_EVENT_CLASS(regmap_bool,
@@ -145,18 +117,10 @@ LTTNG_TRACEPOINT_EVENT_CLASS(regmap_bool,
 
 	TP_ARGS(map, flag),
 
-	TP_STRUCT__entry(
-		__string(	name,		regmap_name(map))
-		__field(	int,		flag		)
-	),
-
-	TP_fast_assign(
-		tp_strcpy(name, regmap_name(map))
-		tp_assign(flag, flag)
-	),
-
-	TP_printk("%s flag=%d", __get_str(name),
-		  (int)__entry->flag)
+	TP_FIELDS(
+		ctf_string(name, regmap_name(map))
+		ctf_integer(int, flag, flag)
+	)
 )
 
 LTTNG_TRACEPOINT_EVENT_INSTANCE(regmap_bool, regmap_cache_only,
@@ -171,6 +135,7 @@ LTTNG_TRACEPOINT_EVENT_INSTANCE(regmap_bool, regmap_cache_bypass,
 	TP_PROTO(struct regmap *map, bool flag),
 
 	TP_ARGS(map, flag)
+
 )
 
 #endif /* LTTNG_TRACE_REGMAP_H */
