@@ -12,28 +12,18 @@
 LTTNG_TRACEPOINT_EVENT(syscall_entry_unknown,
 	TP_PROTO(int id, unsigned long *args),
 	TP_ARGS(id, args),
-	TP_STRUCT__entry(
-		__field(int, id)
-		__array(unsigned long, args, UNKNOWN_SYSCALL_NRARGS)
-	),
-	TP_fast_assign(
-		tp_assign(id, id)
-		tp_memcpy(args, args, UNKNOWN_SYSCALL_NRARGS * sizeof(*args))
-	),
-	TP_printk()
+	TP_FIELDS(
+		ctf_integer(int, id, id)
+		ctf_array(unsigned long, args, args, UNKNOWN_SYSCALL_NRARGS)
+	)
 )
 LTTNG_TRACEPOINT_EVENT(compat_syscall_entry_unknown,
 	TP_PROTO(int id, unsigned long *args),
 	TP_ARGS(id, args),
-	TP_STRUCT__entry(
-		__field(int, id)
-		__array(unsigned long, args, UNKNOWN_SYSCALL_NRARGS)
-	),
-	TP_fast_assign(
-		tp_assign(id, id)
-		tp_memcpy(args, args, UNKNOWN_SYSCALL_NRARGS * sizeof(*args))
-	),
-	TP_printk()
+	TP_FIELDS(
+		ctf_integer(int, id, id)
+		ctf_array(unsigned long, args, args, UNKNOWN_SYSCALL_NRARGS)
+	)
 )
 
 #undef TP_PROBE_CB
@@ -42,32 +32,20 @@ LTTNG_TRACEPOINT_EVENT(compat_syscall_entry_unknown,
 LTTNG_TRACEPOINT_EVENT(syscall_exit_unknown,
 	TP_PROTO(int id, long ret, unsigned long *args),
 	TP_ARGS(id, ret, args),
-	TP_STRUCT__entry(
-		__field(int, id)
-		__field(long, ret)
-		__array(unsigned long, args, UNKNOWN_SYSCALL_NRARGS)
-	),
-	TP_fast_assign(
-		tp_assign(id, id)
-		tp_assign(ret, ret)
-		tp_memcpy(args, args, UNKNOWN_SYSCALL_NRARGS * sizeof(*args))
-	),
-	TP_printk()
+	TP_FIELDS(
+		ctf_integer(int, id, id)
+		ctf_integer(long, ret, ret)
+		ctf_array(unsigned long, args, args, UNKNOWN_SYSCALL_NRARGS)
+	)
 )
 LTTNG_TRACEPOINT_EVENT(compat_syscall_exit_unknown,
 	TP_PROTO(int id, long ret, unsigned long *args),
 	TP_ARGS(id, ret, args),
-	TP_STRUCT__entry(
-		__field(int, id)
-		__field(long, ret)
-		__array(unsigned long, args, UNKNOWN_SYSCALL_NRARGS)
-	),
-	TP_fast_assign(
-		tp_assign(id, id)
-		tp_assign(ret, ret)
-		tp_memcpy(args, args, UNKNOWN_SYSCALL_NRARGS * sizeof(*args))
-	),
-	TP_printk()
+	TP_FIELDS(
+		ctf_integer(int, id, id)
+		ctf_integer(long, ret, ret)
+		ctf_array(unsigned long, args, args, UNKNOWN_SYSCALL_NRARGS)
+	)
 )
 #endif /*  _TRACE_SYSCALLS_UNKNOWN_H */
 
