@@ -64,6 +64,13 @@ void prio_record(struct lttng_ctx_field *field,
 	chan->ops->event_write(ctx, &prio, sizeof(prio));
 }
 
+static
+void prio_get_value(struct lttng_ctx_field *field,
+		union lttng_ctx_value *value)
+{
+	value->s64 = wrapper_task_prio_sym(current);
+}
+
 int lttng_add_prio_to_ctx(struct lttng_ctx **ctx)
 {
 	struct lttng_ctx_field *field;
@@ -92,6 +99,7 @@ int lttng_add_prio_to_ctx(struct lttng_ctx **ctx)
 	field->event_field.type.u.basic.integer.encoding = lttng_encode_none;
 	field->get_size = prio_get_size;
 	field->record = prio_record;
+	field->get_value = prio_get_value;
 	lttng_context_update(*ctx);
 	wrapper_vmalloc_sync_all();
 	return 0;
