@@ -71,6 +71,36 @@ void vppid_record(struct lttng_ctx_field *field,
 	chan->ops->event_write(ctx, &vppid, sizeof(vppid));
 }
 
+static
+void vppid_get_value(struct lttng_ctx_field *field,
+		union lttng_ctx_value *value)
+{
+	struct task_struct *parent;
+	pid_t vppid;
+
+	/*
+	 * current nsproxy can be NULL when scheduled out of exit. pid_vnr uses
+	 * the current thread nsproxy to perform the lookup.
+	 */
+
+	/*
+	 * TODO: when we eventually add RCU subsystem instrumentation,
+	 * taking the rcu read lock here will trigger RCU tracing
+	 * recursively. We should modify the kernel synchronization so
+	 * it synchronizes both for RCU and RCU sched, and rely on
+	 * rcu_read_lock_sched_notrace.
+	 */
+
+	rcu_read_lock();
+	parent = rcu_dereference(current->real_parent);
+	if (!current->nsproxy)
+		vppid = 0;
+	else
+		vppid = task_tgid_vnr(parent);
+	rcu_read_unlock();
+	value->s64 = vppid;
+}
+
 int lttng_add_vppid_to_ctx(struct lttng_ctx **ctx)
 {
 	struct lttng_ctx_field *field;
@@ -92,6 +122,8 @@ int lttng_add_vppid_to_ctx(struct lttng_ctx **ctx)
 	field->event_field.type.u.basic.integer.encoding = lttng_encode_none;
 	field->get_size = vppid_get_size;
 	field->record = vppid_record;
+	field->get_value = vppid_get_value;
+	lttng_context_update(*ctx);
 	wrapper_vmalloc_sync_all();
 	return 0;
 }
